/* 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is forcect
 *
 * The Initial Developer of the Original Code is 
 * Ryan Gammon <rggammon@alumni.uwaterloo.ca>
 * Portions created by Ryan Gammon are Copyright (C) 2003
 * Ryan Gammon. All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 */

const FORCECT_CONTRACTID = "@mozilla.org/js_forcect;1";
const PREF_CONTRACTID = "@mozilla.org/preferences;1";
const ENABLE_ON_STARTUP_PREF = "forcect.enableOnStartup";


include('chrome://jslib/content/io/dirUtils.js');
include('chrome://jslib/content/io/dir.js');
include('chrome://jslib/content/rdf/rdf.js');
include('chrome://jslib/content/rdf/rdfFile.js');

var rdfFile;
var tree;
var tbURIRegexp;
var tbOldCTRegexp;
var tbNewCT;
var nextFilterId;

var sessionEnableCloseVal; // Indicates whether or not we should enable the session
                           // on close

//////// Start hack functions (use jslib internals)

function getResourceFromTree(aTree, aIndex) {
    // XXX: Initialize by hand. This is a hack. jslib needs an attach function
    // XXX: Figure out how to do paths in a generic way
    var currentResource = aTree.builderView.getResourceAtIndex(aIndex);    
    var resource = new RDFResource("node", currentResource.Value, "urn:ForceContentTypeRoot", aTree.database);

    // Swap out the resource
    resource.resource = currentResource;

    return resource;
}

function getSeqFromTree(aTree) {
    return new RDFContainer("seq", "urn:ForceContentTypeRoot", null, aTree.database);
}

///////// End hack functions (use jslib internals)

function focusTree() {
    // XXX: Should be smarter about setting the selection.
    // downloadmanager is an inspiration here -- check it out.
    tree.focus();
    if (tree.view && tree.view.rowCount > 0) {
        tree.treeBoxObject.selection.select(0);
    }    
}

function onSelect(aEvent) {
    var resource = getResourceFromTree(tree, tree.currentIndex);

    tbURIRegexp.value = resource.getAttribute("http://forcect.mozdev.org/rdf#URIRegexp");
    tbOldCTRegexp.value = resource.getAttribute("http://forcect.mozdev.org/rdf#OldCTRegexp");
    tbNewCT.value = resource.getAttribute("http://forcect.mozdev.org/rdf#NewCT");
}

// Called by Update and Add
function updateResource(resource) {
    resource.setAttribute("http://forcect.mozdev.org/rdf#URIRegexp", tbURIRegexp.value);
    resource.setAttribute("http://forcect.mozdev.org/rdf#OldCTRegexp", tbOldCTRegexp.value);
    resource.setAttribute("http://forcect.mozdev.org/rdf#NewCT", tbNewCT.value);
}

function onUpdate() {
    var resource = getResourceFromTree(tree, tree.currentIndex);
    updateResource(resource);
}

function onAdd() {
    var rdfSeq = getSeqFromTree(tree);    
    var newNode = rdfSeq.addNode("FilterRule_" + nextFilterId);
    nextFilterId++;
    updateResource(newNode);    
}


function onRemove() {
    var resource = getResourceFromTree(tree, tree.currentIndex);
    resource.remove();
    focusTree();
}

function onRebuild() {
    focusTree();
}


function onFiltersDialogLoad() {
    // Init globals
    tree = document.getElementById('FiltersTree');
    tbURIRegexp   = document.getElementById('forcectURIRegexp');
    tbOldCTRegexp = document.getElementById('forcectOldCTRegexp');
    tbNewCT       = document.getElementById('forcectNewCT');

    // Set startup pref
    var prefs = Components.classes[PREF_CONTRACTID].getService();
    prefs.QueryInterface(Components.interfaces.nsIPref);
    if(prefs.GetPrefType(ENABLE_ON_STARTUP_PREF) == prefs.ePrefBool) {
        if(prefs.GetBoolPref(ENABLE_ON_STARTUP_PREF)) {
            // set checked
            var cbEnableStartup = document.getElementById('enableStartup');
            cbEnableStartup.checked = true;
        }
    }

    // Set session pref
    var forceCT = Components.classes[FORCECT_CONTRACTID].getService();
    forceCT.QueryInterface(Components.interfaces.nsIForceContentTypeService);
    
    var cbEnableSession = document.getElementById('enableSession');
    cbEnableSession.checked = forceCT.isEnabled;        
    sessionEnableCloseVal = forceCT.isEnabled;        

    // Disable the service while we're working with this dialog
    // Re-enabling will trigger the reloading of the rdf file
    forceCT.disable();

    // Create RDF
    var utils = new DirUtils();

    var profileDir = new Dir(utils.getMozUserHomeDir());
    var file = profileDir.append("forcect.rdf");

    rdfFile = new RDFFile();

    // XXX: My copy of jslib isn't letting me set the namespace in the constructor.
    // This is a hack.
    rdfFile._file_init(file, JS_RDFFILE_FLAG_SYNC, "http://forcect.mozdev.org/rdf#", "FORCECT");

    if(!rdfFile.isValid()) {
        dump("Could not load rdf file\n");
        return;
    }

    // Get the root sequence
    var seq;    
    var list;

    try {
        // JSLIB bug: this always succeeds
        seq = rdfFile.getRootSeq("urn:ForceContentTypeRoot");

        // Any problems will become evident here
        list = seq.getSubNodes();
    }
    catch(ex) {
        seq = null;
        list = null;
    }

    if(!seq) {
        // We must have just created the file. Add an entry for the root sequence
        if(!rdfFile.isValid()) {
            dump("RDF is not valid before call to addRootSeq\n");
        }
        seq = rdfFile.addRootSeq("urn:ForceContentTypeRoot");
        list = seq.getSubNodes();        
    }

    // Find the value of the next filter id
    nextFilterId = 1;    
    for(var i = 0; i < list.length; i++) {
        var nodeSubject = list[i].getSubject();        
        nodeSubject.match(/urn:ForceContentTypeRoot:FilterRule_(\d+)/);
        var num = parseInt(RegExp.$1);

        if(nextFilterId <= num) {
            nextFilterId = num + 1;
        }
    }

    tree.database.AddDataSource(rdfFile.getDatasource());
    tree.builder.rebuild();

    window.setTimeout(onRebuild, 0);
}

function onClose() {
    // free the globals
    rdfFile = null;
    tbURIRegexp = null;
    tbOldCTRegexp = null;
    tbNewCT = null;
    nextFilterId = null;

    window.close();

    // set service state
    var forceCT = Components.classes[FORCECT_CONTRACTID].getService();
    forceCT.QueryInterface(Components.interfaces.nsIForceContentTypeService);

    if(sessionEnableCloseVal) {
        forceCT.enable();
    }   
    else {
        forceCT.disable();
    }   
}

function onOk() {
    var prefs = Components.classes[PREF_CONTRACTID].getService();
    prefs.QueryInterface(Components.interfaces.nsIPref);

    rdfFile.flush();

    cbEnableStartup = document.getElementById('enableStartup');
    prefs.SetBoolPref(ENABLE_ON_STARTUP_PREF, cbEnableStartup.checked);

    var forceCT = Components.classes[FORCECT_CONTRACTID].getService();
    forceCT.QueryInterface(Components.interfaces.nsIForceContentTypeService);

    cbEnableSession = document.getElementById('enableSession');
    sessionEnableCloseVal = cbEnableSession.checked;

    onClose();
}

function onCancel() {
    onClose();
}

function onFiltersDialogUnload() {
}
