/* 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is forcect
 *
 * The Initial Developer of the Original Code is 
 * Ryan Gammon <rggammon@alumni.uwaterloo.ca>
 * Portions created by Ryan Gammon are Copyright (C) 2003
 * Ryan Gammon. All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 */

const FORCECT_CID = Components.ID('{5f9369d7-bfd0-4530-9417-2637393b29ae}');
const FORCECT_CONTRACTID = "@mozilla.org/js_forcect;1";
const OBSERVERSERVICE_CONTRACTID = "@mozilla.org/observer-service;1";
const LOADER_CONTRACTID = "@mozilla.org/moz/jssubscript-loader;1";
const CATMAN_CONTRACTID = "@mozilla.org/categorymanager;1";
const PREF_CONTRACTID = "@mozilla.org/preferences;1";

// Our preferences
const ENABLE_ON_STARTUP_PREF = "forcect.enableOnStartup";

function nsForceContentTypeService() {
}

var nsForceContentTypeService = {
    
    // Begin nsIHttpHeaderVisitor implementation
    visitHeader : function (aHeader, aValue) {
        // Useful for debugging...
        //   oHttp.QueryInterface(Components.interfaces.nsIRequest);
        //   oHttp.visitRequestHeaders(this);        
        //   oHttp.visitResponseHeaders(this);        

        dump("Got header: " + aHeader + " " + aValue + "\n");
    },
    // End nsIHttpHeaderVisitor implementation

    // Begin nsIObserver implementation
    jslibInitialized : false,
    observe: function(aSubject, aTopic, aData) {
        if (aTopic == 'app-startup') {
            if(!this.jslibInitialized) {
                // Load jslib. 
                // XXX: We could actually do this in something like loadRdf() instead of
                // using an app-startup listener (I had it that way originally, but moved
                // it here while debugging an rdf problem [missing the sync flag])
                dump("Loading jslib from forcectService app-startup\n");
                var jsLoader = Components.classes[LOADER_CONTRACTID].createInstance(Components.interfaces.mozIJSSubScriptLoader);  
                jsLoader.loadSubScript("chrome://jslib/content/jslib.js");
        
                // include function is defined by jslib, which we just loaded
                include('chrome://jslib/content/io/dirUtils.js');
                include('chrome://jslib/content/io/dir.js');
                include('chrome://jslib/content/rdf/rdf.js');
                include('chrome://jslib/content/rdf/rdfFile.js');

                // app-startup seems to be called more than once. Set a flag so that
                // we only try to initialize jslib once.
                this.jslibInitialized = true;
            }
        } 
        else if (aTopic == 'http-on-examine-merged-response' || 
                 aTopic == 'http-on-examine-response') {
            if(this.filters) {
                var oHttp = aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);

                if(aTopic == 'http-on-examine-merged-response') {
                    dump('This is a cached document\n');
                }

                for(var i = 0; i < this.filters.length; i++) {
                    dump("Matching against " + this.filters[i].uriRegexp + "\n");
                    dump(oHttp.URI.spec + " " + oHttp.contentType + "\n");
                    if(this.filters[i].uriRegexp.test(oHttp.URI.spec) &&
                       this.filters[i].oldCTRegexp.test(oHttp.contentType)) {
                           dump("Changing Content-Type to " + this.filters[i].newCT + "\n");
                           oHttp.contentType = this.filters[i].newCT;
                           break;
                       }
                }
            }
        }

        else {
            dump("nsForceContentTypeService: unknown TOPIC: " + aTopic + "\n");
        }
    },
    // End nsIObserver implementation

    // Begin nsIForceContentTypeService implementation
    isEnabled : false, // Exposed as a readonly attribute
    filters : null,
    isInitialized: false,

    init: function() {
        if(!this.isInitialized) {
            var prefs = Components.classes[PREF_CONTRACTID].getService();
            prefs.QueryInterface(Components.interfaces.nsIPref);
            if(prefs.GetPrefType(ENABLE_ON_STARTUP_PREF) == prefs.ePrefInvalid) {
                dump("Setting pref (init)\n");
                prefs.SetBoolPref(ENABLE_ON_STARTUP_PREF, false);
            }
            else {
                if(prefs.GetBoolPref(ENABLE_ON_STARTUP_PREF)) {
                    this.enable();
                }
            }            
            this.isInitialized = true;
        }
    },

    enable: function() {
        // Register listener
        if(!this.isEnabled) {
            // Get config information
            this.loadRdf();

            var observerService = Components.classes[OBSERVERSERVICE_CONTRACTID].getService(Components.interfaces.nsIObserverService);
            observerService.addObserver(nsForceContentTypeService, "http-on-examine-merged-response", false);
            observerService.addObserver(nsForceContentTypeService, "http-on-examine-response", false);
            this.isEnabled = true;
            dump("Enabled forcect service\n");
        }
    },

    disable: function() {
        if(this.isEnabled) {
            var observerService = Components.classes[OBSERVERSERVICE_CONTRACTID].getService(Components.interfaces.nsIObserverService);
            observerService.removeObserver(nsForceContentTypeService, "http-on-examine-merged-response");
            observerService.removeObserver(nsForceContentTypeService, "http-on-examine-response");
            this.isEnabled = false;
            dump("Disabled forcect service\n");
        }
    },

    loadRdf: function() {
        this.filters = new Array();
                
        var utils = new DirUtils();

        var profileDir = new Dir(utils.getMozUserHomeDir());
        var file = profileDir.append("forcect.rdf");

        // XXX: rdf will cache a failure to load the RDF file if we use the 
        // DONT_CREATE flag here, and future calls will fail too, even if they
        // don't have the DONT_CREATE flag set. In a perfect world, jslib would
        // try to refresh a datasource when it tries to load it. For now, work 
        // around this by not using the DONT_CREATE flag
        //rdfFile = new RDFFile(file, JS_RDFFILE_FLAG_DONT_CREATE | JS_RDFFILE_FLAG_SYNC);

        rdfFile = new RDFFile(file, JS_RDFFILE_FLAG_SYNC);
        if(rdfFile.isValid()) {
            // Get the root sequence
            var seq;    
            var nodes;
            
            try {
                // JSLIB doesn't currently check to see if the root seq actually exists.
                // This always succeeds (bug)
                seq = rdfFile.getRootSeq("urn:ForceContentTypeRoot");

                // This will throw an exception if the root seq didn't actually exist.
                // Not elegant, but jslib isn't doing any of the "HasAssertion" stuff.
                nodes = seq.getSubNodes();
            }
            catch(ex) {
                // We're missing the sequence.
                seq = null;
                nodes = null;
            }

            if(nodes) {
                for(var i = 0; i < nodes.length; i++) {
                    var filter = { uriRegexp: null, oldCTRegexp: null, newCT: null };
                    filter.uriRegexp   = new RegExp(nodes[i].getAttribute("http://forcect.mozdev.org/rdf#URIRegexp"));
                    filter.oldCTRegexp = new RegExp(nodes[i].getAttribute("http://forcect.mozdev.org/rdf#OldCTRegexp"));
                    filter.newCT       = nodes[i].getAttribute("http://forcect.mozdev.org/rdf#NewCT");
                    
                    dump("Adding " + filter.uriRegexp + " " + filter.oldCTRegexp + " " + filter.newCT + "\n");

                    this.filters.push(filter);
                }
            }
        }
    },

    // End nsIForceContentTypeService implementation

    // Begin nsISupports implementation
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(Components.interfaces.nsIHttpHeaderVisitor) &&
            !iid.equals(Components.interfaces.nsIHttpNotify) &&
            !iid.equals(Components.interfaces.nsIForceContentTypeService) &&
            !iid.equals(Components.interfaces.nsIObserver)) {
                // dump("forcectService: QI unknown interface: " + iid + "\n");
                throw Components.results.NS_ERROR_NO_INTERFACE;
            }
        return this;
    }
    // End nsISupports implementation
};

// nsForceContentType Class Factory
var nsForceContentTypeFactory = {
    createInstance: function(outer, iid) {
        if (outer != null) {
            throw Components.results.NS_ERROR_NO_AGGREGATION;
        }
    
        // XXX: Is it cool to throw NS_ERROR_NO_INTERFACE via QI on failure?
        // Some components check iid's here, and return NS_ERROR_INVALID_ARG instead
        return nsForceContentTypeService.QueryInterface(iid);
    }
}


// nsForceContentType Module (for XPCOM registration) 
var nsForceContentTypeModule = {
    registerSelf: function(compMgr, fileSpec, location, type) {
        // We don't need to defer registration (as seen in nsSample.js)

        // dump("Registering forcect\n");
        var compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(FORCECT_CID,
                                        "Force Content-Type JS component",
                                        FORCECT_CONTRACTID,
                                        fileSpec, location, type);


        var catman = Components.classes[CATMAN_CONTRACTID].getService(Components.interfaces.nsICategoryManager);
        catman.addCategoryEntry("app-startup", "ForceCT", FORCECT_CONTRACTID, true, true);
    },

    unregisterSelf: function(compMgr, fileSpec, location) {
        // When does unregister ever get called, anyway?
        dump("UNREGISTERING forcect\n");
        var compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(FORCECT_CID, fileSpec);

        var catman = Components.classes[CATMAN_CONTRACTID].getService(Components.interfaces.nsICategoryManager);
        catman.deleteCategoryEntry("app-startup", FORCECT_CONTRACTID, true);
    },

    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(FORCECT_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return nsForceContentTypeFactory;
    },

    canUnload: function(compMgr) { 
        return true; 
    }
};

/* module initialization */
function NSGetModule(comMgr, fileSpec) { 
    return nsForceContentTypeModule; 
}

